/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <sys/stat.h>
#include <string.h>
#include "va_assert.h"
#include "va_log.h"
#include "varid.h"
#include "sam2pretty_lib.h"
#include <errno.h>


void split_reads(char **filenames,
		 SnpGroups_t * snpgroup,
		 VA_INT32 ** fid_reads, VA_Vector fastaVec) {
	VA_ASSERT(fid_reads != NULL);

	FILE ** fids = (FILE **) malloc(sizeof(FILE *) * (snpgroup->size));
	VA_ASSERT(fids != NULL);

	/* PRECONDITION: snp groups don't overlap in ranges */

	VA_INT32 * f_reads = (VA_INT32 *) malloc(sizeof(VA_INT32) * (snpgroup->size + 1));
	VA_ASSERT(f_reads != NULL);
	memset(f_reads, 0, (snpgroup->size + 1) * sizeof(VA_INT32));

	int i, j;
	for (i = 0; i < snpgroup->size; i++) {
		if (filenames[i] != NULL) {
			fids[i] = fopen(filenames[i], "w");
			if (fids[i] == NULL) {
				perror("FATAL IO Error");
				exit(1);
			}
			VA_ASSERT(fids[i] != NULL);
		} else {
			fids[i] = NULL;
		}
	}


	int totreads = 0;
	int usedreads = 0;

	/* go through each line and re-write it in the right file */
	/* get the line (with newline char) */
	char line[BUFSIZ];
	while (fgets(line, BUFSIZ, alignments_file) != NULL) {
		if (line[0] == '@') {
			continue;
		}

		//Found a read
		totreads++;
		
		//find the contig name, skip first two tabs
		if (predict_start==0) {
			int i; int length=strlen(line);
			for(i=0; i<length && line[i]!='\t'; i++); i++;
			for(; i<length && line[i]!='\t'; i++); i++;
			if (i>=length) {
				fprintf(stderr,"A fatal error has occured in parsing SAM input!\n");
				exit(1);
			}
			char * rname = line+i;
			/* determine where this read belongs */
			for (j = 0; j < snpgroup->size; j++) {
				VA_ASSERT(filenames[j]!=NULL);
				SnpGroup_t *s = &snpgroup->data[j];
				int min_length=MIN(s->chr.size,strlen(rname));
				//if (s->chr.size!=strlen(rname)) {
				//	continue;
				//}
				if (strncmp(rname,(char*)s->chr.data,min_length)==0) {
					pretty * pa = pretty_from_string(line);
					if (pa!=NULL) {
						pretty * head = pretty_sub_prettys(pa);
						while ( head != NULL ) {
							pretty_ls(head);
							pretty_qualities(head);
							if (pa->colour_space) {
								pretty_cs(head);
							}
							pretty_print_sam(fids[j],head);
							pretty * temp=(pretty*)head->next;
							if (head==pa) {
								pa=NULL;
							}
							pretty_free(head);
							head=temp;
							f_reads[j]++;
							usedreads++;
						}
						if (pa!=NULL) {
							pretty_free(pa);
						}
					}
					break;
				}
			}
		} else {
			pretty * pa=pretty_from_string(line);
			if (pa!=NULL) {
				int res = pretty_remap(pa,hmm_start,hmm_end);
				if (res==1) {
					/* determine where this read belongs */
					for (j = 0; j < snpgroup->size; j++) {
						VA_ASSERT(filenames[j]!=NULL);
						SnpGroup_t *s = &snpgroup->data[j];
						int min_length=MIN(s->chr.size,strlen(pa->reference_name));
						//if (s->chr.size!=strlen(pa->reference_name)) {
						//	continue;
						//}

						if (strncmp(pa->reference_name,(char*)s->chr.data,min_length)==0) {
							//fprintf(fids[j],"%s",line);
							pretty * head = pretty_sub_prettys(pa);
							while ( head != NULL ) {
								pretty_ls(head);
								pretty_qualities(head);
								if (pa->colour_space) {
									pretty_cs(head);
								}
								pretty_print_sam(fids[j],head);
								pretty * temp=(pretty*)head->next;
								if (head==pa) {
									pa=NULL;
								}
								pretty_free(head);
								head=temp;
								f_reads[j]++;
								usedreads++;
							}
							if (pa!=NULL) {
								pretty_free(pa);
							}
							break;
						}
					}
				} else {
					pretty_free(pa);
				}
			}
		}
	}

	for (i = 0; i < snpgroup->size; i++) {
		if (filenames[i] != NULL)
			fclose(fids[i]);
	}
	free(fids);
	/* oprint some useful information */
	fprintf(stderr, "\tTotal reads: %d\n", totreads);
	fprintf(stderr, "\tUsed reads: %d\n", usedreads);

	*fid_reads = f_reads;
}
